@@ -41,11 +41,11 @@
 static ssize_t iovec_read(
 		int conn, char *buf, size_t buflen, struct int_window *fds)
 {
-	char cmsgdata[(CMSG_LEN(MAX_LIBWAY_FDS * sizeof(int32_t)))];
+	char cmsgdata[(CMSG_LEN(MAX_LIBWAY_FDS * sizeof(int32_t)))] = {0};
 	struct iovec the_iovec;
 	the_iovec.iov_len = buflen;
 	the_iovec.iov_base = buf;
-	struct msghdr msg;
+	struct msghdr msg = {0};
 	msg.msg_name = NULL;
 	msg.msg_namelen = 0;
 	msg.msg_iov = &the_iovec;
@@ -54,6 +54,9 @@ static ssize_t iovec_read(
 	msg.msg_controllen = sizeof(cmsgdata);
 	msg.msg_flags = 0;
 	ssize_t ret = recvmsg(conn, &msg, 0);
+	if (msg.msg_flags & MSG_CTRUNC) {
+		wp_error("Warning, control data was truncated in recvmsg");
+	}
 
 	// Read cmsg
 	struct cmsghdr *header = CMSG_FIRSTHDR(&msg);
@@ -145,7 +148,7 @@ static int translate_fds(struct fd_translation_map *map,
 			size_t fdsz = 0;
 			enum fdcat fdtype = get_fd_type(fds[i], &fdsz);
 			sfd = translate_fd(map, render, fds[i], fdtype, fdsz,
-					NULL, false, false);
+					NULL, false);
 		}
 		if (sfd) {
 			ids[i] = sfd->remote_id;
@@ -726,8 +729,9 @@ static int advance_waymsg_chanwrite(struct way_msg_state *wmsg,
 							&wmsg->ack_msgs[i]) {
 				not_in_prog_msg = &wmsg->ack_msgs[1 - i];
 			}
-			if (wmsg->transfers.vecs[next_slot].iov_base ==
-					&wmsg->ack_msgs[i]) {
+			if (next_slot < wmsg->transfers.end &&
+					wmsg->transfers.vecs[next_slot].iov_base ==
+							&wmsg->ack_msgs[i]) {
 				queued_msg = &wmsg->ack_msgs[i];
 			}
 		}
@@ -872,7 +876,7 @@ static int advance_waymsg_progread(struct way_msg_state *wmsg,
 		if (rc == -1 && (errno == EWOULDBLOCK || errno == EAGAIN)) {
 			// do nothing
 		} else if (rc == 0 || (rc == -1 && errno == ECONNRESET)) {
-			wp_error("%s has closed", progdesc);
+			wp_debug("%s has closed", progdesc);
 			// state transitions handled in main loop
 			return ERR_STOP;
 		} else if (rc == -1) {
