@@ -372,9 +372,11 @@ enum fdcat get_fd_type(int fd, size_t *size)
 					fsdata.st_mode);
 		}
 		return FDC_PIPE;
-	} else if (is_dmabuf(fd) == 1) {
-		return FDC_DMABUF;
 	} else {
+		/* Note: we cannot at the moment reliably identify a dmabuf;
+		 * trying to do so by importing it may file if we have the wrong
+		 * parameters.
+		 */
 		wp_error("The fd %d has an unusual mode %x (type=%x): blk=%d chr=%d dir=%d lnk=%d reg=%d fifo=%d sock=%d; expect an application crash!",
 				fd, fsdata.st_mode, fsdata.st_mode & S_IFMT,
 				S_ISBLK(fsdata.st_mode),
@@ -529,7 +531,7 @@ static void uncompress_buffer(struct thread_pool *pool, struct comp_ctx *ctx,
 struct shadow_fd *translate_fd(struct fd_translation_map *map,
 		struct render_data *render, int fd, enum fdcat type,
 		size_t file_sz, const struct dmabuf_slice_data *info,
-		bool read_modifier, bool force_pipe_iw)
+		bool force_pipe_iw)
 {
 	struct shadow_fd *sfd = get_shadow_for_local_fd(map, fd);
 	if (sfd) {
@@ -647,8 +649,7 @@ struct shadow_fd *translate_fd(struct fd_translation_map *map,
 				sizeof(struct dmabuf_slice_data));
 		init_render_data(render);
 		sfd->dmabuf_bo = import_dmabuf(render, sfd->fd_local,
-				&sfd->buffer_size, &sfd->dmabuf_info,
-				read_modifier);
+				&sfd->buffer_size, &sfd->dmabuf_info);
 		if (!sfd->dmabuf_bo) {
 			return sfd;
 		}
@@ -665,8 +666,7 @@ struct shadow_fd *translate_fd(struct fd_translation_map *map,
 		// TODO: multifd-dmabuf video surface
 		init_render_data(render);
 		sfd->dmabuf_bo = import_dmabuf(render, sfd->fd_local,
-				&sfd->buffer_size, &sfd->dmabuf_info,
-				read_modifier);
+				&sfd->buffer_size, &sfd->dmabuf_info);
 		if (!sfd->dmabuf_bo) {
 			return sfd;
 		}
@@ -679,15 +679,10 @@ struct shadow_fd *translate_fd(struct fd_translation_map *map,
 		sfd->buffer_size = 0;
 
 		init_render_data(render);
-		if (info) {
-			memcpy(&sfd->dmabuf_info, info,
-					sizeof(struct dmabuf_slice_data));
-		} else {
-			// already zero initialized (no information).
-		}
+		memcpy(&sfd->dmabuf_info, info,
+				sizeof(struct dmabuf_slice_data));
 		sfd->dmabuf_bo = import_dmabuf(render, sfd->fd_local,
-				&sfd->buffer_size, &sfd->dmabuf_info,
-				read_modifier);
+				&sfd->buffer_size, &sfd->dmabuf_info);
 		if (!sfd->dmabuf_bo) {
 			return sfd;
 		}
@@ -1577,8 +1572,7 @@ int apply_update(struct fd_translation_map *map, struct thread_pool *threads,
 			return 0;
 		}
 
-		sfd->dmabuf_bo = make_dmabuf(
-				render, sfd->buffer_size, &sfd->dmabuf_info);
+		sfd->dmabuf_bo = make_dmabuf(render, &sfd->dmabuf_info);
 		if (!sfd->dmabuf_bo) {
 			sfd->fd_local = -1;
 			return 0;
@@ -1640,8 +1634,7 @@ int apply_update(struct fd_translation_map *map, struct thread_pool *threads,
 			sfd->fd_local = -1;
 			return 0;
 		}
-		sfd->dmabuf_bo = make_dmabuf(
-				render, sfd->buffer_size, &sfd->dmabuf_info);
+		sfd->dmabuf_bo = make_dmabuf(render, &sfd->dmabuf_info);
 		if (!sfd->dmabuf_bo) {
 			wp_error("FDC_DMAVID_IW: RID=%d make_dmabuf failure, sz=%d (%d)",
 					sfd->remote_id, (int)sfd->buffer_size,
@@ -1711,8 +1704,7 @@ int apply_update(struct fd_translation_map *map, struct thread_pool *threads,
 			return 0;
 		}
 
-		sfd->dmabuf_bo = make_dmabuf(
-				render, sfd->buffer_size, &sfd->dmabuf_info);
+		sfd->dmabuf_bo = make_dmabuf(render, &sfd->dmabuf_info);
 		if (!sfd->dmabuf_bo) {
 			wp_error("FDC_DMAVID_IR: RID=%d make_dmabuf failure",
 					sfd->remote_id);
@@ -2293,8 +2285,7 @@ void read_readable_pipes(struct fd_translation_map *map)
 	}
 }
 
-void extend_shm_shadow(struct fd_translation_map *map,
-		struct thread_pool *threads, struct shadow_fd *sfd,
+void extend_shm_shadow(struct thread_pool *threads, struct shadow_fd *sfd,
 		size_t new_size)
 {
 	if (sfd->buffer_size >= new_size) {
@@ -2315,7 +2306,6 @@ void extend_shm_shadow(struct fd_translation_map *map,
 	}
 
 	increase_buffer_sizes(sfd, threads, new_size);
-	(void)map;
 
 	// leave `sfd->remote_bufsize` unchanged, and mark dirty
 	sfd->is_dirty = true;
