@@ -636,7 +636,6 @@ static void configure_low_latency_enc_context(struct AVCodecContext *ctx,
 	ctx->max_b_frames = 0; // Q: how to get this to zero?
 	// low latency
 	ctx->delay = 0;
-	ctx->thread_count = 1;
 
 	if (sw) {
 		ctx->bit_rate = bpf * nom_fps;
@@ -667,10 +666,6 @@ static void configure_low_latency_enc_context(struct AVCodecContext *ctx,
 		if (fmt == VIDEO_H264) {
 			/* with i965/gen8, hardware encoding is faster but has
 			 * significantly worse quality per bitrate than x264 */
-			if (av_opt_set(ctx->priv_data, "quality", "7", 0) !=
-					0) {
-				wp_error("Failed to set h264 encode quality");
-			}
 			if (av_opt_set(ctx->priv_data, "profile", "main", 0) !=
 					0) {
 				wp_error("Failed to set h264 encode main profile");
@@ -1000,7 +995,6 @@ int setup_video_decode(struct shadow_fd *sfd, struct render_data *rd)
 	}
 
 	ctx->delay = 0;
-	ctx->thread_count = 1;
 	if (has_hw) {
 		/* If alignment permits, use hardware decoding */
 		has_hw = pad_hardware_size((int)sfd->dmabuf_info.width,
